/************************************************************
 * Template-Klasse fuer Referenz-Semantik
 *  - stellt sicher, dass bei Vergleichen die Elemente
 *    und nicht die Zeiger verglichen werden
 ************************************************************/
template <class T>
class Ptr {
  private:
    T* adr;      // eigentlicher Zeiger auf den Wert

  public:
    /* Konstruktor aus Wert vom Typ T
     *  - merkt sich Adresse des Wertes
     */
    Ptr (T& t)
     : adr(&t) {
    }

    // Dereferenzierung
    T& operator*() const {
        return *adr;
    }
    T* operator->() const {
        return adr;
    }

    // Vergleiche
    friend bool operator== (const Ptr<T>& lhs, const Ptr<T>& rhs);
    friend bool operator<  (const Ptr<T>& lhs, const Ptr<T>& rhs);
};

// Implementierung der Vergleiche
template <class T>
inline bool operator== (const Ptr<T>& lhs, const Ptr<T>& rhs)
{
    return *lhs.adr == *rhs.adr;
}
template <class T>
inline bool operator< (const Ptr<T>& lhs, const Ptr<T>& rhs)
{
    return *lhs.adr < *rhs.adr;
}
